package uebung02;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.time.LocalDateTime;

/**
 * 
 * @author lorenz schauer
 * 
 * Konto-Klasse
 * Mit Schreibfunktion auf Log-Datei, die entsprechend angelegt wird, falls sie nicht existiert. 
 *
 */
public class Konto {

	private String kontonummer;
	private double kontostand;
	
	// Feldvariablen fr File, Lese- und Schreibe Streams
	private File logFile;
	private BufferedReader reader = null;
	private BufferedWriter writer = null;
	
	// Feldvriable fr die eigenen Filter
	private EncodingWriter encode = null;
	private DecodingReader decode = null;
	
	// Konstruktor
	public Konto(String kontonummer, double kontostand) {
		super();
		this.kontonummer = kontonummer;
		this.kontostand = kontostand;
		
		// Log-Datei:
		logFile = new File("log-File-ktn_"+kontonummer+".txt");
		if(!logFile.exists()){		
			try {
				// anlegen, falls nicht existiert.
				logFile.createNewFile();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	public void einzahlen(double betrag){
		
		this.kontostand+=betrag;
		
		// Nachdem der Betrag eingezahlt wurde erfolgt Eintrag in die Logfile:
		try {
			writer = new BufferedWriter(new FileWriter(logFile,true)); // Eintrag wird dan Inhalt angehngt
			
			// Filter instanziieren und den erzeugten BufferedWriter bergeben
			encode = new EncodingWriter(writer);
			
			// Jetzt die berschriebene write-Methode des Filters nutzen. 
			encode.write(LocalDateTime.now().toString()+": Einzahlung auf Konto "+this.kontonummer+": "+betrag+" Euro.\n");
			
			// Nicht vergessen: Beide Streams schlieen. 
			encode.close();
			writer.close();
		} catch (IOException e) {
			e.printStackTrace();
		}finally{
			if (writer!=null)
				try {
					encode.close();
					writer.close(); // Nicht vergessen Stream auf jeden Fall zu schlieen!
				} catch (IOException e) {
					e.printStackTrace();
				}
		}
		
	}
	
	public void auszahlen(double betrag){
		this.kontostand-=betrag;
		
		// Nachdem der Betrag ausgezahlt wurde erfolgt Eintrag in die Log-File:
		try {
			writer = new BufferedWriter(new FileWriter(logFile,true));
			
			// Analog zur einzahlen-Methode
			encode = new EncodingWriter(writer);
				
			encode.write(LocalDateTime.now().toString()+": Auszahlung vom Konto "+this.kontonummer+": "+betrag+" Euro.\n");
			
			encode.close();
			writer.close();
		} catch (IOException e) {
			e.printStackTrace();
		}finally{
			if (writer!=null)
				try {
					encode.close();
					writer.close(); 
				} catch (IOException e) {
					e.printStackTrace();
				}
		}
	}
	
	
	public String getKontonummer() {
		return kontonummer;
	}

	public double getKontostand() {
		return kontostand;
	}
	
	// Methode zum Anzeigen der Log-Datei ohne vorherige Entschlsselung
	public void seeLogFile(){
		
		System.out.println("Printing Log-File undecoded...");
		try {
			reader = new BufferedReader(new FileReader(this.logFile));
			String line; 
			while((line=reader.readLine())!=null){
				System.out.println(line);
			}
			reader.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}finally{
			if(reader!=null){
				try {
					reader.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
	}
	
	// Methode zum Anzeigen der Log-Datei mit vorheriger Entschlsselung
	public void decodeLogFile(){
		
		System.out.println("Printing decoded Log-File");
		try {
			reader = new BufferedReader(new FileReader(this.logFile));
			
			// FilterReader instanziieren und den BufferedReader mitgeben. 
			decode = new DecodingReader(reader);
			
			int c;
			// Zeichenweise auselesen, solange bis kein Zeichen mehr da ist (bis -1 zurckgegeben wird)
			while((c = decode.read())>=0)
				System.out.print((char) c);
			
			// Wieder schlieen
			decode.close();
			reader.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}finally{
			if(reader!=null){
				try {
					decode.close();
					reader.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}		
		
		
	}
	
}
